/******************************************************************************
Filename:     modSysSimpleImpl.c
Target:       TLN13SP01

Description:
building block layer module library: system part

ϵͳʼʱӿʵ֡

modification:
2015.01.25 creation Zheng Hui
           WM_SDK_G1.02.12\wm_main.cmain.cֲ

Zheng Hui, Guangzhou, China. hzheng@gzhu.edu.cn. Guangzhou University
******************************************************************************/

#include "modSys.h"
#include "wm_osal.h"
#include "wm_irq.h"
#include "wm_hostspi.h"
#include "wm_flash.h"
#include "wm_fls_gd25qxx.h"
#include "wm_params.h"
#include "wm_debug.h"
#include "wm_fwup.h"
#include "wm_efuse.h"
#include "wm_netif.h"
#include "tls_sys.h"
#include "wm_cmdp_hostif.h"  //App\wm_atcmd
#include "wm_uart_task.h"    //App\wm_atcmd
#include "wm_netif.h"
#include "wm_wifi_oneshot.h"

/**
 * 
 */
#define     TASK_START_STK_SIZE         300     /* Size of each task's stacks (# of WORDs)  */


#define FW_MAJOR_VER           0x01
#define FW_MINOR_VER           0x02
#define FW_PATCH_VER           0x12

const char FirmWareVer[4] = {
	'G', 
	FW_MAJOR_VER,  /* Main version */ 
	FW_MINOR_VER, /* Subversion */
	FW_PATCH_VER  /* Internal version */
	};
const char HwVer[6] = {
	'H',
	0x1,
	0x0,
	0x0,
	0x0,
	0x0
};


/**
 * ȫֱ
 */
//for c library mutext
tls_os_sem_t    *libc_sem;
#if (TLS_OS_FREERTOS && configUSE_HEAP3)
static u8 malloc_no = 0;
#endif


extern u8 tx_gain_group[];
u32 tls_debug_level = 0xf;


OS_STK      TaskStartStk[TASK_START_STK_SIZE];         /* Tasks stacks */

/**
 * 
 */
void task_start (void *data);
extern void *tls_wl_init(u8 *tx_gain, u8* mac_addr);
extern int wpa_supplicant_init(u8 *mac_addr);
extern void tls_sys_auto_mode_run(void);

/**
 * ΢ʼӿ
 */
int8 modSysInit() { 
	
#if TLS_OS_UCOS
#elif TLS_OS_FREERTOS
	tls_irq_init();
//	tls_os_sem_create(&libc_sem, 1);
#endif	

#if TLS_OS_UCOS
    /* Initialize uC/OS-II   */
    OSInit();

    /* before use malloc() function, must create mutex used by c_lib */
    tls_os_sem_create(&libc_sem, 1);

#endif		
	
	//create startup task
	tls_os_task_create(NULL, NULL,
                    task_start,
                    (void *)0,
                    (void *)TaskStartStk,          /* ջʼַ */
                    TASK_START_STK_SIZE * sizeof(u32), /* ջĴС     */
                    1,
                    0);
	
  
  return 0;
}

/**
 * ΢ϵͳѭ
 */
int8 modSysRun() { 
#if TLS_OS_UCOS
    OSStart();
#elif TLS_OS_FREERTOS
	vTaskStartScheduler();
#endif
  return 0;
}

/**
 * ʱӿڣ
 * 
 *     vMilliSecond: ʱ
 * עʱʱ䲻ȷ
 */
int8 modDelay(uint16 vMilliSecond){
  u32 timer_ticks;

  //timer_ticks = (vMilliSecond * OS_TMR_CFG_TICKS_PER_SEC) / OS_TICKS_PER_SEC;
	timer_ticks = (vMilliSecond * OS_TICKS_PER_SEC) / 1000;
  if (timer_ticks == 0)
    timer_ticks = 1;
		
  tls_os_time_delay(timer_ticks);
  return 0;
}



/*****************************************************************************
 * Function Name        // task_start
 * Descriptor             // before create multi_task, we create a task_start task
 *                      	   // in this example, this task display the cpu usage
 * Input                
 * Output              
 * Return              
 ****************************************************************************/
void task_start (void *data)
{
    extern void CreateUserTask(void);
    extern void  RestoreParamToDefault(void);
    int err = 0;
    u8 mac_addr[6];
   
#if TLS_OS_UCOS
	tls_irq_init();
    /* initial os ticker */
    tls_os_timer_init(); 

#if OS_TASK_STAT_EN > 0
    /* Initialize uC/OS-II's statistics */
    OSStatInit();

#endif
#endif
	
    //read saved parameters
    tls_spi_slave_sel(SPI_SLAVE_FLASH);	
    tls_spi_init();
    tls_fls_init();
    tls_fls_gd25qxx_drv_install();
	  tls_restore_param_from_backup();	

    err = tls_param_init(); 
    if (err) {
        TLS_DBGPRT_INFO("read spi param err\n"); 
    }
    tls_fwup_init();
		
		//mac address init

    /* ȡefuseеmacַ */
   	tls_get_mac_addr(mac_addr);
    TLS_DBGPRT_INFO("mac addr ");
    TLS_DBGPRT_DUMP((char *)(&mac_addr[0]), 6);
    /* ȡefuseеtx_gain */
	  tls_get_tx_gain(tx_gain_group);    
    TLS_DBGPRT_INFO("tx gain ");
    TLS_DBGPRT_DUMP((char *)(&tx_gain_group[0]), 12);

    if(tls_wl_init(NULL, &mac_addr[0]) == NULL){ //set tx_gain and mac_addr
        TLS_DBGPRT_INFO("wl driver initial failured\n");
    }
    if (wpa_supplicant_init(mac_addr)) {
        TLS_DBGPRT_INFO("supplicant initial failured\n");
    }
		
		//ethernet init
    tls_ethernet_init();
    //sys init
    tls_sys_init();

		//host control interface init
		//use uart0 for control
    tls_hostif_init();
    tls_uart_init();

   	tls_sys_auto_mode_run(); 
		

		//enable wifi on shot config
	  tls_wifi_one_shot_init();

	  /* User start here */
	  CreateUserTask();
    for (;;) {
	    tls_os_time_delay(0x10000000);
    }
}


/**
 * OS specified function
 */

#if TLS_OS_UCOS
void  OSTaskReturnHook        (OS_TCB          *ptcb)
{
    (void)ptcb;
}
#endif
#if !TLS_OS_UCOS
//???os_cpu_a.s????
void  OSIntExit (void)
{}

void  OSTaskSwHook (void)
{
}
#endif


/* c librayr mutex */
/*----------------------------------------------------------------------------
 *      Standard Library multithreading interface
 *---------------------------------------------------------------------------*/

#ifndef __MICROLIB
/*--------------------------- _mutex_initialize -----------------------------*/

int _mutex_initialize (u32 *mutex) {
    /* Allocate and initialize a system mutex. */
    //tls_os_sem_create(&libc_sem, 1);
    //mutex = (u32 *)libc_sem;
    return(1);
}


/*--------------------------- _mutex_acquire --------------------------------*/

void _mutex_acquire (u32 *mutex) {
    //u8 err;
    /* Acquire a system mutex, lock stdlib resources. */
#if 	(TLS_OS_FREERTOS && configUSE_HEAP3)
	if(malloc_no < 2)
		malloc_no ++;
	else
#endif		
  		tls_os_sem_acquire(libc_sem, 0);
}


/*--------------------------- _mutex_release --------------------------------*/

void _mutex_release (u32 *mutex) {
    /* Release a system mutex, unlock stdlib resources. */
#if 	(TLS_OS_FREERTOS && configUSE_HEAP3)
	if(2 == malloc_no)
		malloc_no ++;
	else if(malloc_no > 2)
#endif		
    		tls_os_sem_release(libc_sem);
}

#endif


/**
 * ʱӿڣ
 * 
 *     vMicroSecond: ʱ΢
 * ʵּCPUʱΪ80MHz
 * עʱʱ䲻ȷ
 */

int8 modDelayUs(uint8 vMicroSecond){
	u32  cpu_sr;
	cpu_sr = tls_os_set_critical();
	#pragma arm
	while(vMicroSecond--)	{
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");
		asm("NOP");

	}	
	tls_os_release_critical(cpu_sr);
  return 0;
}
