/******************************************************************************
Filename:     modUartSimpleImpl.c
Target:       TLN13SP01

Description:
building block layer module library: UART part

UART0 used as ATCMD host interface, managed by WM_SDK\App\wm_atcmd module.

UART1 managed here to be user data interface. Use my frame format.

Since it's intended for communication with CC2530, the packet size should be less
than 120.

modification:
2015.01.25 creation Zheng Hui
           modified from CC2530 modUartSimpleImpl.c

Zheng Hui, Guangzhou, China. hzheng@gzhu.edu.cn. Guangzhou University

******************************************************************************/
#include "modUart.h"

#include "wm_uart.h"
#include "wm_osal.h"
#include <string.h>                     /* memset(), memcpy() */


/**
 * 
 */
#define MOD_UART_MAX_PDU_SIZE  120
#define     TASK_STK_SIZE      256
#define     UART1_TASK_PRIO    60
#define     UART1_TASK_QUEUE_SIZE	32

//task message
#define	MSG_UART_RECEIVE_DATA			1


/**
 * ȫֱ
 */
//for tls_uart_write()
extern struct tls_uart_port uart_port[2];
 
//UART global FIFO Buffer
/*
//tx buffer not used
#define UART_TX_BUFFER_LEN 360
uint8 gUart1TxBuffer[UART_TX_BUFFER_LEN];
uint16 gUart1TxStartPointer=0;
uint16 gUart1TxEndPointer=0;
uint8 gUart1SendingFlag=0;
*/
#define UART_RX_BUFFER_LEN 360 
uint8 gUart1RxBuffer[UART_RX_BUFFER_LEN];
uint16 gUart1RxStartPointer=0;
uint16 gUart1RxEndPointer=0;
uint16 gUart1RxCounter = 0;

#define UART_RX_MAX_FRAME_LEN 256 
uint8 gUart1RxFrameBuffer[UART_RX_MAX_FRAME_LEN];

//for task
OS_STK      mUart1TaskStack[TASK_STK_SIZE];         /* Tasks stacks */
tls_os_queue_t *mUart1TaskQueuePtr = NULL;
void *mUart1TaskQueue[UART1_TASK_QUEUE_SIZE];


/**
 * 
 */
void modUartDataProcTask(void *data);
s16 modUartRxCallback(char *buf, u16 len); //lower level rx callback
	
void modUart1RxFramingRoutine(void);

//for upper level
static modUartRxCallbackFunction* mUartRxUsrCallBackFunction=0;


/**
 * 
 */
uint16 modUart1GetRxFreeBufferSize(void);
int8 modUart1PushRxFIFO(unsigned char vData);
unsigned char modUart1PopRxFIFO(void);



/**
 * òʼ
 */
int8 modUartInit(uint8 vUartNumber, uint32 vBaudrate){
	//support UART1 only
  if (vUartNumber!=1) {
    return -3;
  }
	
	//start uart1 rx task	
	tls_os_queue_create(&mUart1TaskQueuePtr,
            &mUart1TaskQueue[0],
            UART1_TASK_QUEUE_SIZE, 0);
	
  tls_os_task_create(NULL, NULL,
                    modUartDataProcTask,
                    (void *)0,
                    (void *)mUart1TaskStack,      
                    TASK_STK_SIZE * sizeof(u32),
                    UART1_TASK_PRIO,
                    0);	

  //config uart1 as user mode
	tls_uart_cfg_user_mode();
	tls_user_uart_rx_register( modUartRxCallback);
	tls_user_uart_set_baud_rate(vBaudrate);


  return 0;

}
/**
 * uart1 data handling task
 */
void modUartDataProcTask(void *data) {
	void *pMsg;
	
	for(;;) {
		tls_os_queue_receive(mUart1TaskQueuePtr, (void **)&pMsg, 0, 0);
		switch((u32)pMsg) {
			case MSG_UART_RECEIVE_DATA:
				modUart1RxFramingRoutine(); //handle received data
				break;
			default:
				break;
		}
	}
}

/**
 * lower layer Uart RX callback
 */
s16 modUartRxCallback(char *buf, u16 len) {
	uint16 i;
	//copy data to FIFO buffer
	if (len>modUart1GetRxFreeBufferSize()) {
		return -1;
	}
	for (i=0;i<len;i++) {
		modUart1PushRxFIFO((uint8)buf[i]);
	}
	
	//send msg to modUartDataProcTask
	tls_os_queue_send(mUart1TaskQueuePtr,(void *)MSG_UART_RECEIVE_DATA,0);
	
	//
	return 0;
}


/**
 * Uart RX framing routine
 * identified frame data is put into gUart1RxFrameBuffer
 */
#define MOD_UART_RX_FLAG_IDLE 0
#define MOD_UART_RX_FLAG_SDF1 1
#define MOD_UART_RX_FLAG_SDF2 2
#define MOD_UART_RX_FLAG_LEN  3
#define MOD_UART_RX_FLAG_DATA 4
static uint8 gModUart1RxFlag=0; //FSM flag
static uint8 gModUart1RxDataRemain=0;
static uint8 gModUart1RxPktLen=0;

void modUart1RxFramingRoutine() {
  uint8 iInData;
  //
	while (gUart1RxEndPointer != gUart1RxStartPointer) {
		//pop 1 byte from FIFO
		iInData = modUart1PopRxFIFO();
		
		switch (gModUart1RxFlag) {
		case MOD_UART_RX_FLAG_IDLE:
			if (iInData == 0x7e) {
				gModUart1RxFlag = MOD_UART_RX_FLAG_SDF1;
			}//else ignore
			break;
		case MOD_UART_RX_FLAG_SDF1:
			if (iInData == 0x7e) {
				gModUart1RxFlag = MOD_UART_RX_FLAG_SDF2;
			}else {//lost sync
				gModUart1RxFlag = MOD_UART_RX_FLAG_IDLE;
			}
			break;
		case MOD_UART_RX_FLAG_SDF2:
			//iInData is packet len
			gModUart1RxDataRemain = iInData;
			gModUart1RxFlag = MOD_UART_RX_FLAG_DATA;
			gModUart1RxPktLen= iInData;
			break;
		case MOD_UART_RX_FLAG_DATA:
			gUart1RxFrameBuffer[gModUart1RxPktLen-gModUart1RxDataRemain] = iInData;
			gModUart1RxDataRemain --;
			if (gModUart1RxDataRemain == 0) {
				//receive one whole packet
				//call user routine
				if(mUartRxUsrCallBackFunction) {
					mUartRxUsrCallBackFunction(0, gModUart1RxPktLen, gUart1RxFrameBuffer);
				}
				//
				gModUart1RxFlag = MOD_UART_RX_FLAG_IDLE;
			}
			break;
		}
	}

}

/**
 * ýշصӿ
 * appģڸúʵֶԷĴ
 */
int8 modUartSetRxCallback(modUartRxCallbackFunction* vFunction) {
  mUartRxUsrCallBackFunction = vFunction;
  return 0;
}



/**
 * ͷӿ
 */
/**
 * ʽ
 * Flag(0x7E) Flag(0x7E) LEN(1byte, unsigned) data
 */
int8 modUartSendPkt(uint8 vUartNumber, uint8 vSize, const uint8 *vPacket) {
	uint8 pktHeader[3];
	struct tls_uart_port *port = &uart_port[1];

  if (vUartNumber != 1) {
    return -1;
  }
	
	pktHeader[0]= 0x7e;
	pktHeader[1]= 0x7e;
	pktHeader[2]= vSize;
	
	//push header bytes one by one
	while (tls_uart_write(port,(char *)pktHeader,1)!=1){
	}
	while (tls_uart_write(port,(char *)pktHeader+1,1)!=1){
	}
	while (tls_uart_write(port,(char *)pktHeader+2,1)!=1){
	}
	//send packet body
	tls_uart_tx_sync((char *)vPacket,vSize);

  return 0;
}



/**
 * ȡײʵֵ֧飨ûݣ
 */
uint8 modUartGetMaxPduSize() {
  return MOD_UART_MAX_PDU_SIZE;
}



/**
 * check Buffer free space
 */
uint16 modUart1GetRxFreeBufferSize() {
  if (gUart1RxStartPointer == gUart1RxEndPointer) {
    return UART_RX_BUFFER_LEN-1;
  }else if (gUart1RxStartPointer<gUart1RxEndPointer) {
    uint16 iTemp = gUart1RxEndPointer - gUart1RxStartPointer-1;
    return iTemp;
  }else {
    uint16 iTemp =  UART_RX_BUFFER_LEN - (gUart1RxStartPointer - gUart1RxEndPointer)-1;
    return iTemp;
  }
}

/**
 * write uart RX FIFO
 */
int8 modUart1PushRxFIFO(unsigned char vData) {
  uint16 iNewStartPointer = gUart1RxStartPointer + 1;
  if (iNewStartPointer >=UART_RX_BUFFER_LEN) {
    iNewStartPointer = 0;
  }
  if (iNewStartPointer == gUart1RxEndPointer) { //faster than hardware TX
    return -1; //drop new data, return error
  }
  gUart1RxBuffer[gUart1RxStartPointer] = vData;
  gUart1RxStartPointer=iNewStartPointer;
  return 0;
}

/**
 * read uart RX FIFO
 */
unsigned char modUart1PopRxFIFO() {
  unsigned char ret;
  if (gUart1RxEndPointer == gUart1RxStartPointer) {
    return 0;
  }
  ret = gUart1RxBuffer[gUart1RxEndPointer];
  //
  gUart1RxEndPointer ++;

  if (gUart1RxEndPointer >=UART_RX_BUFFER_LEN) {
    gUart1RxEndPointer = 0;
  }
  return ret;
}


