
/*+**********************************************************************************
  Filename: main.c

  Description: 
	  main function of application.
		
		A simple app toggles leds via user task and software delay.

  Modification:
    2015.01.25 creation by Zheng Hui

**********************************************************************************-*/
#include <stdio.h>
#include "wm_osal.h"
#include "wm_netif.h"

#include "modSys.h"
#include "modHwExt.h"
#include "modUart.h"

#include "../include/modUdp.h"

/**
 * 
 */
#define     TASK_STK_SIZE         256     /* Size of each task's stacks (# of WORDs)  */
#define     MY_TASK_PRIO      50
/**
 * ȫֱ
 */
OS_STK      mMyLedToggleTaskStack[TASK_STK_SIZE];         /* Tasks stacks */
uint8 mModNetstatusFlag = 0;

/**
 * 
 */
void myLedToggleTask(void *data);
void myUartRxCallbackFunction(uint8 vUartNumber, uint8 vSize, uint8* vPacket); //Uart Rx Callback
static void modNetStatusChangedEventCb(u8 status );

/**
 * 
 */
int main( void ) {

  modSysInit();

  modSysRun();

}



/**
 * user app init
 */
void CreateUserTask(void) {
	uint8 pWellcomeMsg[] = "uart ready";
	
	//app init
	//
  modLedInit();
	//register network status callback
	mModNetstatusFlag = 0;
	tls_netif_add_status_event(modNetStatusChangedEventCb);

	//
	modUartInit(1,115200);
	modUartSetRxCallback(myUartRxCallbackFunction);
	modUartSendPkt(1, 11, pWellcomeMsg);
	//
	modUdpConnInit();
	
	//
	//start led control task
		tls_os_task_create(NULL, NULL,
                    myLedToggleTask,
                    (void *)0,
                    (void *)mMyLedToggleTaskStack,      
                    TASK_STK_SIZE * sizeof(u32),
                    MY_TASK_PRIO,
                    0);


}


/**
 * UART Rx Callback
 */
void myUartRxCallbackFunction(uint8 vUartNumber, uint8 vSize, uint8* vPacket){
	//send it to UDP
	modSendUdpPacket(vPacket, vSize);
}

/**
 * user app task
 */
void myLedToggleTask(void *data) {
	for (;;) { //loop
		if (mModNetstatusFlag == 0) {
			modSetLedState(0,0);
			modSetLedState(1,1);
			modDelay(500);
			modSetLedState(1,0);
			modDelay(500);
		}else if (mModNetstatusFlag == 1) {
			modSetLedState(0,1);
			modSetLedState(1,1);
			modDelay(500);
			modSetLedState(1,0);
			modDelay(500);
		}else {
			modSetLedState(0,1);
			//modSetLedState(1,0);
			modDelay(1000);
		}
	
	}
}
/**
 * network status listener
 */
static void modNetStatusChangedEventCb(u8 status ) {
	switch(status) {
		case NETIF_WIFI_DISCONNECTED:
		case NETIF_WIFI_JOIN_FAILED:
			mModNetstatusFlag = 0;
			break;
		case NETIF_WIFI_JOIN_SUCCESS:
			mModNetstatusFlag = 1;
		case NETIF_IP_NET_UP:
			mModNetstatusFlag = 2;
			break;
		default:
			break;
	}
}
