
/*+**********************************************************************************
  Filename: modWifiUdpMulticastConn.c

  Description: 
	  exchange message between uart and a UDP multicast connection.
		
  Modification:
    2015.01.28 creation by Zheng Hui

**********************************************************************************-*/
#include <stdio.h>
#include <string.h>

#include "wm_osal.h"
#include "wm_netif.h"
#include "wm_sockets.h"
#include "wm_mem.h"
#include "wm_cmdp.h"

#include "modSys.h"
#include "modHwExt.h"
#include "modUart.h"

/**
 * 
 */
#define     TASK_STK_SIZE         256     /* Size of each task's stacks (# of WORDs)  */
#define     UDP_TASK_PRIO      52

#define MOD_MCAST_PORT 5200
#define MOD_MCAST_BUF_SIZE	1024

#define MOD_AP_SSID_LEN 7
#define MOD_AP_PWD_LEN 12
static uint8 mModApSSID[]="ZH_HOME";
static uint8 mModApPwd[]="jeffreyzheng";

/**
 * ȫֱ
 */
OS_STK      mModUdpTaskStack[TASK_STK_SIZE];         /* Tasks stacks */

int mModUdpSocket=0;
u8 mModMCastIP[4] = {224,1,2,1};
struct sockaddr_in mModMultiAddr;

//extern uint8 mModNetstatusFlag;

/**
 * 
 */
void modUdpConnTask(void *data);
void handleUdpPacket(uint8 *vBuffer, int vLen, struct sockaddr_in vFromAddr, int vAddrLen);

/**
 * init
 */
int modUdpConnInit( void ) {
	//struct tls_cmd_connect_t sConn;
	u8 automode = WIFI_AUTO_CNT_ON;
  //set auto connect
	tls_wifi_auto_connect_flag( WIFI_AUTO_CNT_FLAG_SET, &automode);
	
	//
	mModMultiAddr.sin_family=AF_INET;
  mModMultiAddr.sin_port=htons(MOD_MCAST_PORT);
  MEMCPY((char *)&mModMultiAddr.sin_addr, (char *)mModMCastIP, 4);
	
	//join wifi network
	//tls_cmd_join(CMD_MODE_INTERNAL, &sConn);
	
	//create udp connection control task
  tls_os_task_create(NULL, NULL,
                    modUdpConnTask,
                    (void *)0,
                    (void *)mModUdpTaskStack,      
                    TASK_STK_SIZE * sizeof(u32),
                    UDP_TASK_PRIO,
                    0);
										
  return 0;
}

/**
 * user app task
 */
void modUdpConnTask(void *data) {
  struct sockaddr_in sLocalAddr,sFromAddr;
  struct ip_mreq sMReq;
	int ret;
  int ttl=10;
  int loop=0;
	//uint8 pBuffer[MOD_MCAST_BUF_SIZE];
	uint8 *pBuffer;
	socklen_t iSocklen;
	struct tls_ethif *pEthif; 
	
	//init
  pBuffer = tls_mem_alloc(MOD_MCAST_BUF_SIZE);
  if(NULL == pBuffer)
	  return;
	
  memset(pBuffer,0,MOD_MCAST_BUF_SIZE);

	
  sLocalAddr.sin_family=AF_INET;
  sLocalAddr.sin_port=htons(MOD_MCAST_PORT);
  sLocalAddr.sin_addr.s_addr=htonl(0x00000000UL);
  //
	sFromAddr.sin_family=AF_INET;
  sFromAddr.sin_port=htons(MOD_MCAST_PORT);
  sFromAddr.sin_addr.s_addr=htonl(0x00000000UL);
  //
	MEMCPY((char *)&sMReq.imr_multiaddr.s_addr, (char *)mModMCastIP, 4);
  sMReq.imr_interface.s_addr=htonl(0x00000000UL);
	/*
  //debug
	for (;;) { //loop
		
    if (mModNetstatusFlag == 2) {
			modSetLedState(1,2);
		}
		
		modDelay(1000);
	}//dead loop
	*/
	//
	
	//join network
	ret = tls_wifi_connect(mModApSSID, MOD_AP_SSID_LEN, mModApPwd, MOD_AP_PWD_LEN);

	
	while (ret <0) {
		modDelay(1000);
		ret = tls_wifi_connect(mModApSSID, MOD_AP_SSID_LEN, mModApPwd, MOD_AP_PWD_LEN);
	}
	


	for (;;) { //loop
		modDelay(1000);
	  pEthif= tls_netif_get_ethif();
    if (pEthif->status == 1) {			
      modSetLedState(1,0);			
			//new udp socket
			mModUdpSocket=socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);
			if (mModUdpSocket<=0) {
				printf("socket error");
				//mModNetstatusFlag = 0;
				continue;
			}
			ret=bind(mModUdpSocket,(struct sockaddr*)&sLocalAddr,sizeof(sLocalAddr));
			if(ret<0){
				printf("bind error");
				//mModNetstatusFlag = 0;
				continue;
			}	
			//init dest ipaddrr
			
			//set socket multicast option
			if(setsockopt(mModUdpSocket,IPPROTO_IP,IP_MULTICAST_TTL,&ttl,sizeof(ttl))<0){
				printf("setsockopt IP_MULTICAST_TTL failed");
				//mModNetstatusFlag = 0;
				continue;
			}
			if(setsockopt(mModUdpSocket,IPPROTO_IP,IP_MULTICAST_LOOP,&loop,sizeof(loop))<0){
				printf("setsockopt IP_MULTICAST_LOOP failed");
				//mModNetstatusFlag = 0;
				continue;
			}
			if(setsockopt(mModUdpSocket,IPPROTO_IP,IP_ADD_MEMBERSHIP,&sMReq,sizeof(sMReq))<0){
				printf("setsockopt IP_ADD_MEMBERSHIP failed");
				//mModNetstatusFlag = 0;
				continue;
			}
			//
			modSetLedState(1,1);
			
			//loop
			while (1) {
				//wait for incoming pkt
				iSocklen = sizeof(sFromAddr);
				ret = recvfrom(mModUdpSocket,pBuffer,MOD_MCAST_BUF_SIZE,0,(struct sockaddr*)&sFromAddr, &iSocklen);
				//
				if (ret <0 ) {
					//something wrong
					//mModNetstatusFlag = 0;
					break;
				}
				//handle pkt
				if (ret >0) {
					handleUdpPacket(pBuffer, ret, sFromAddr, iSocklen);
				}
				//check for signal to leave
			}//loop recvfrom
			//socket error
			setsockopt(mModUdpSocket,IPPROTO_IP,IP_DROP_MEMBERSHIP,&sMReq,sizeof(sMReq));
			closesocket(mModUdpSocket);	
			mModUdpSocket = 0;

		}//if NETIF_IP_NET_UP
		if (mModUdpSocket>0) {
			closesocket(mModUdpSocket);	
			mModUdpSocket = 0;			
		}
		//something wrong with the connection
	}//dead loop

}


/**
 * app specified process
 */
void handleUdpPacket(uint8 *vBuffer, int vLen, struct sockaddr_in vFromAddr, int vAddrLen) {
	
	//just transfer data to Uart
	modUartSendPkt(1, vLen, vBuffer);
	//
	modSetLedState(1,2);
}

/**
 * send udp packet
 */
void modSendUdpPacket(uint8 *vBuffer, int vLen) {
	if (mModUdpSocket == 0) {
	  return;
	}
	//send packet
	sendto(mModUdpSocket,vBuffer,vLen,0,(struct sockaddr*)&mModMultiAddr,sizeof(mModMultiAddr));
	//
	modSetLedState(1,2);
}

